#!/bin/sh

DO_CBCTL=0
DO_TWISTCLI=0
DO_OSSPI=0
DO_K8S=0
DO_BD=0
SCAN_IMAGE_LIST=""
APP_NAME=""
BLACKDUCK_PROJECT_NAME=""
BLACKDUCK_PROJECT_VERSION=""
BLACKDUCK_PROJECT_GROUP=""
BLACKDUCK_DETECT_TOOLS=""
BLACKDUCK_EXCLUDED_TOOLS=""
BLACKDUCK_TARGET=""

WORST_EXIT_CODE=0

usage() {
	printf "\n"
	echo "Usage: $0 [-c | -t | -o | -k] -s SCAN_IMAGE_LIST"
	printf "\t-c\tfor cbctl scanning\n"
	printf "\t-t\tfor twistcli scanning\n"
	printf "\t-o\tfor osspi scanning\n"
	printf "\t-k\tenable kubernetes helm chart scan\n"
	printf "\t-a\tapplication name for kubernetes helm chart scan\n"
	printf "\t-b\tfor BlackDuck scan\n"
	printf "\tYou can also combine several options\n"
	printf "\t-s\tquoted space separated list of images to scan\n"
	printf "\n"
	echo "Example: $0 -c -t -o -s \"artifactory/image1 artifactory/image2 harbor.vmware.com/image3\""
	printf "\n"
	exit 1
}

######################
# Check script usage #
######################

while getopts "ctobka:s:" options; do
	case "${options}" in
	c)
		DO_CBCTL=1
		;;
	t)
		DO_TWISTCLI=1
		;;
	o)
		DO_OSSPI=1
		;;
	b)
		DO_BD=1
		;;
	k)
		DO_K8S=1
		;;
	a)
		APP_NAME=${OPTARG}
		;;
	s)
		SCAN_IMAGE_LIST=${OPTARG}
		;;
	:)
		echo "Error: -${OPTARG} requires an argument."
		usage
		;;
	*)
		usage
		;;
	esac
done

if test $((DO_CBCTL + DO_TWISTCLI + DO_OSSPI + DO_K8S + DO_BD)) -eq 0; then
	echo "Error: at least 1 scan should be performed"
	usage
fi

if test -z "${SCAN_IMAGE_LIST}" && [ $DO_K8S -eq 0 ] && [ $DO_BD -eq 0 ]; then
	echo "Error: at least 1 container image should be provided for scanning"
	usage
fi

####################
# Invoke the scans #
####################

if test ${DO_CBCTL} -eq 1; then
	# Configure cbctl
	sh script/generateConfig_cbctl.sh
	cp generated/.cbctl.yaml "${HOME}/.cbctl/"

	export SCAN_IMAGE_LIST
	sh scan_cbctl.sh
	SCAN_EXIT_CODE=$?
	WORST_EXIT_CODE=$((WORST_EXIT_CODE > SCAN_EXIT_CODE ? WORST_EXIT_CODE : SCAN_EXIT_CODE))
fi

if test ${DO_TWISTCLI} -eq 1; then
	export SCAN_IMAGE_LIST
	sh scan_twistcli.sh
	SCAN_EXIT_CODE=$?
	WORST_EXIT_CODE=$((WORST_EXIT_CODE > SCAN_EXIT_CODE ? WORST_EXIT_CODE : SCAN_EXIT_CODE))
fi

if test ${DO_OSSPI} -eq 1; then
	export SCAN_IMAGE_LIST
	sh osspi/scan_osspi.sh
	SCAN_EXIT_CODE=$?
	WORST_EXIT_CODE=$((WORST_EXIT_CODE > SCAN_EXIT_CODE ? WORST_EXIT_CODE : SCAN_EXIT_CODE))
fi

if test ${DO_K8S} -eq 1; then
	sh script/generateConfig_cbctl.sh
	cp generated/.cbctl.yaml "${HOME}/.cbctl/"
	export APP_NAME
	python3 kubernetes/helm_chart_scanner.py
	SCAN_EXIT_CODE=$?
	WORST_EXIT_CODE=$((WORST_EXIT_CODE > SCAN_EXIT_CODE ? WORST_EXIT_CODE : SCAN_EXIT_CODE))
fi

if test ${DO_BD} -eq 1; then
	if [ -z "${BLACKDUCK_TARGET}" ] && [ -n "${SCAN_IMAGE_LIST}" ]; then
		BLACKDUCK_TARGET="${SCAN_IMAGE_LIST}"
	fi

	echo "Inside the start.sh file"
	echo "group: ${BLACKDUCK_PROJECT_GROUP}"
	echo "name: ${BLACKDUCK_PROJECT_NAME}"
	echo "version: ${BLACKDUCK_PROJECT_VERSION}"
	echo "target: ${BLACKDUCK_TARGET}"

	python3 blackduck/scanner.py -pn "${BLACKDUCK_PROJECT_NAME}" \
		-pv "${BLACKDUCK_PROJECT_VERSION}" \
		-pg "${BLACKDUCK_PROJECT_GROUP}" \
		-dt "${BLACKDUCK_DETECT_TOOLS}" \
		-et "${BLACKDUCK_EXCLUDED_TOOLS}" \
		-t "${BLACKDUCK_TARGET}"

	SCAN_EXIT_CODE=$?
fi

#########################
# After-scan activities #
#########################

# Generate the final html report
sh html_report.sh

# Upload results to S3
if test -n "${AWS_SECPROD_KEY_ID}" -a -n "${AWS_SECPROD_SECRET}" -a -n "${AWS_SECPROD_CIS_BUCKET}" -a -n "${AWS_SECPROD_CIS_BUCKET_REGION}"; then
	python3 s3_uploader/s3_uploader.py -rp "${REP_PATH_CLI}" -bn "${AWS_SECPROD_CIS_BUCKET}"
fi

exit "${WORST_EXIT_CODE}"
